@@ -1,6 +1,6 @@
 from a2ml.api.base_a2ml import BaseA2ML
 from a2ml.api.utils.show_result import show_result
-from a2ml.api.utils import convert_source
+#from a2ml.api.utils import convert_source
 from a2ml.api.utils.context import Context
 
 
@@ -28,7 +28,7 @@ class A2ML(BaseA2ML):
         self.local_runner = lambda: self.build_runner(ctx, provider, force_local=True)
 
     @show_result
-    def import_data(self, source=None):
+    def import_data(self, source=None, name=None, description=None):
         """Imports data defined in context. Uploading the same file name will result in versions being appended to the file name.
 
         Note:
@@ -36,11 +36,18 @@ class A2ML(BaseA2ML):
 
             .. code-block:: yaml
 
-                # Local file name or remote url to the data source file
+                # Local file name, remote url to the data source file or postgres url
                 source: './dataset.csv'
 
+            .. code-block:: yaml
+
+                # Postgres url parameters: dbname, tablename, offset(OPTIONAL), limit(OPTIONAL)
+                source: jdbc:postgresql://user:pwd@ec2-54-204-21-226.compute-1.amazonaws.com:5432/dbname?tablename=table1&offset=0&limit=100
+
         Args:
-            source (str, optional): Local file name or remote url to the data source file or Pandas DataFrame
+            source (str, optional): Local file name, remote url to the data source file, Pandas DataFrame or postgres url
+            name (str, optional): Name of dataset, if none then file name used. If source is DataFrame then name should be specified.
+            description (str, optionsl): Description of dataset
 
         Returns:
             Results for each provider. ::
@@ -65,8 +72,62 @@ class A2ML(BaseA2ML):
                 a2ml = A2ML(ctx, 'auger, azure')
                 a2ml.import_data()
         """
-        with convert_source(source, self.ctx.config.get("name", "source_data")) as data_source:
-            return self.runner.execute('import_data', source=data_source)
+        return self.runner.execute('import_data', source=source, name=name, description=description)
+
+    @show_result
+    def preprocess_data(self, data, preprocessors, locally=False):
+        """Preprocess data
+
+        Args:
+            data (str|pandas.DataFrame): Input data for preprocess. Can be path to file(local or s3) or Pandas Dataframe
+            preprocessors (array of dicts): List of preprocessors with parameters ::
+
+                [
+                    {'text': {'text_cols': []}}
+                ]
+
+        Preprocessors:
+            text
+                * text_cols(array): List of text columns to process
+                * text_metrics ['mean_length', 'unique_count', 'separation_score'] : Calculate metrics for text fields and after vectorize(separation_score) 
+                * tokenize (dict): Default - {'max_text_len': 30000, 'tokenizers': ['sent'], 'remove_chars': '○•'}
+                * vectorize ('en_use_lg'|'hashing'|'en_use_md'|'en_use_cmlm_md'|'en_use_cmlm_lg'): See see https://github.com/MartinoMensio/spacy-universal-sentence-encoder
+                * dim_reduction(dict): Generate features based on vectors. See https://scikit-learn.org/stable/modules/generated/sklearn.decomposition.PCA.html ::
+
+                    {
+                        'alg_name': 'PCA'|'t-SNE', 
+                        'args': {'n_components': 2} #Number of components to keep.
+                    }
+
+                * output_prefix (str): Prefix for generated columns. Format name: {prefix}_{colname}_{num}
+
+                * calc_distance ['none', 'cosine', 'cityblock', 'euclidean', 'haversine', 'l1', 'l2', 'manhattan', 'nan_euclidean'] | 'cosine' : See https://scikit-learn.org/stable/modules/generated/sklearn.metrics.pairwise.distance_metrics.html#sklearn.metrics.pairwise.distance_metrics
+                * compare_pairs (array of dicts): When calc_distance is not none. ::
+
+                    [
+                        {'compare_cols': [{'dataset_idx': 0, 'cols': ['col1']}, {'dataset_idx': 1, 'cols': ['col2']}], 
+                            'output_name':'cosine_col1_col2', 'params': {}
+                        },
+                        {'compare_cols': [{'dataset_idx': 0, 'cols': ['col3']}, {'dataset_idx': 1, 'cols': ['col4']}], 
+                            'output_name':'cosine_col3_col4', 'params': {}
+                        },                        
+                    ]
+
+                * datasets: List of datasets to process, may be empty, so all fields takes from main dataset ::
+
+                    [
+                        {'path': 'path', 'keys': ['main_key', 'local_key'], 'text_metrics': ['separation_score', 'mean_length', 'unique_count']}, 
+                        {'path': 'path1', 'keys': ['main_key1', 'local_key1']}
+                    ]
+
+        Returns:
+                {
+                    'result': True,
+                    'data': 'data in input format'
+                }
+
+        """
+        return self.get_runner(locally).execute_one_provider('preprocess_data', data, preprocessors, locally)
 
     @show_result
     def train(self):
@@ -170,7 +231,8 @@ class A2ML(BaseA2ML):
         return self.runner.execute('evaluate', run_id = run_id)
 
     @show_result
-    def deploy(self, model_id, locally=False, review=True, provider=None):
+    def deploy(self, model_id, locally=False, review=False, provider=None,
+            name=None, algorithm=None, score=None, data_path=None, metadata=None):
         """Deploy a model locally or to specified provider(s).
 
         Note:
@@ -178,10 +240,15 @@ class A2ML(BaseA2ML):
             This method support only one provider
 
         Args:
-            model_id (str): The model id from any experiment you will deploy.
+            model_id (str): The model id from any experiment you will deploy. Ignored for 'external' provider
             locally(bool): Deploys the model locally if True, on the Provider Cloud if False. The default is False.
             review(bool): Should model support review based on actual data. The default is True.
             provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.
+            name (str): Friendly name for the model. Used as name for Review Endpoint
+            algorithm (str): Monitored model(external provider) algorithm name.
+            score (float): Monitored model(external provider) score.
+            data_path (str): Data path to fit model when deploy. Return new deployed model-id
+            metadata (dict): Additional parameter for the model. Used for accurcay report(report parameter)
 
         Returns:
             ::
@@ -196,13 +263,23 @@ class A2ML(BaseA2ML):
 
                 ctx = Context()
                 a2ml = A2ML(ctx, 'auger, azure')
-                a2ml.deploy(model_id='A017AC8EAD094FD')
+                a2ml.deploy(model_id='A017AC8EAD094FD', name='FirstExperiment')
+
+            .. code-block:: python
+
+                ctx = Context()
+                a2ml = A2ML(ctx, 'external')
+                result = a2ml.deploy(model_id=None, name="My external model.", algorithm='RandomForest', score=0.75)
+                model_id = result['data']['model_id']
+
         """
-        return self.get_runner(locally, model_id, provider).execute_one_provider('deploy', model_id, locally, review)
+        return self.get_runner(locally, model_id, provider).execute_one_provider('deploy',
+            model_id, locally, review, name, algorithm, score, data_path, metadata)
 
     @show_result
-    def predict(self, model_id, filename=None, data=None, columns=None, predicted_at=None, 
-            threshold=None, output=None, locally=False, provider=None):
+    def predict(self, model_id, filename=None, data=None, columns=None, predicted_at=None,
+            threshold=None, score=False, score_true_data=None,
+            output=None, no_features_in_result = None, locally=False, provider=None):
         """Predict results with new data against deployed model. Predictions are stored next to the file with data to be predicted on. The file name will be appended with suffix _predicted.
 
         Note:
@@ -211,13 +288,16 @@ class A2ML(BaseA2ML):
 
         Args:
             model_id(str): The deployed model id you want to use.
-            filename(str): The file with data to request predictions for.            
-            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFramme to_dict method
+            filename(str): The file with data to request predictions for.
+            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFrame to_dict('list') method
             columns(list): list of column names if data is array of records
             predicted_at: Predict data date. Use for review of historical data.
             threshold(float): For classification models only. This will return class probabilities with response.
+            score(bool): Calculate scores for predicted results.
+            score_true_data(str, pandas.DataFrame, dict): Data with true values to calculate scores. If missed, target from filename used for true values.
             output(str): Output csv file path.
-            locally(bool): Predicts using a local model if True, on the Provider Cloud if False.
+            no_features_in_result(bool) : Do not return feature columns in prediction result. False by default
+            locally(bool, str): Predicts using a local model with auger.ai.predict if True, on the Provider Cloud if False. If set to "docker", then docker image used to run the model
             provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider set in costructor or config.
 
         Returns:
@@ -267,37 +347,63 @@ class A2ML(BaseA2ML):
                 # if rv[provider].result is True
                 # predictions are returned as rv[provider]['data']['predicted']
 
+            .. code-block:: python
+                
+                # Predict locally without config files. Model will automatically downloaded if not exists.
+                # To use local predict install a2ml[predict]
+                ctx = Context()
+                ctx.config.set('name', 'project name')
+                ctx.credentials = "Json string from a2ml ui settings"
+
+                rv = A2ML(ctx).predict(model_id, '../irises.csv',
+                    no_features_in_result = True, locally=True)
+                # if rv[provider].result is True
+                # predictions are stored in rv[provider]['data']['predicted']
+
         """
-        return self.get_runner(locally, model_id, provider).execute_one_provider('predict', filename, model_id, threshold, locally, data, columns, predicted_at, output)
+        return self.get_runner(locally, model_id, provider).execute_one_provider('predict', filename, model_id, 
+            threshold, locally, data, columns, predicted_at, output, no_features_in_result,
+            score, score_true_data )
 
     @show_result
-    def actuals(self, model_id, filename=None, data=None, columns=None, actuals_at=None, actual_date_column=None, locally=False, provider=None):
+    def actuals(self, model_id, filename=None, data=None, columns=None, actuals_at=None, 
+            actual_date_column=None, experiment_params=None, locally=False, provider=None):
         """Submits actual results(ground truths) for predictions of a deployed model. This is used to review and monitor active models.
 
         Note:
             It is assumed you have predictions against this model first.
 
             .. list-table:: actuals.csv
-                :widths: 50 50
+                :widths: 50 50 50
                 :header-rows: 1
 
-                * - target: predicted value
+                * - predicted( or target): predicted value. If missed - predict called automatically
                   - actual
+                  - baseline_target: predicted value for baseline model (OPTIONAL)
                 * - Iris-setosa
                   - Iris-setosa
+                  - Iris-setosa
                 * - Iris-virginica
                   - Iris-virginica
-                * It may also contain train features to retrain while Review
+                  - Iris-virginica
+
+            It may also contain train features to predict(if target missed), retrain model while Review and for distribution chart
 
             This method support only one provider
 
         Args:
             model_id(str): The deployed model id you want to use.
             filename(str): The file with data to request predictions for.
-            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFramme to_dict method
+            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFrame to_dict('list') method
             columns(list): list of column names if data is array of records
             actuals_at: Actuals date. Use for review of historical data.
             actual_date_column(str): name of column in data which contains actual date
+            experiment_params(dict): parameters to calculate experiment metrics ::
+
+                start_date(date): experiment actuals start date
+                end_date(date):  experiment actuals end date
+                date_col(str): column name with date
+
             locally(bool): Process actuals locally.
             provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider set in costructor or config.
 
@@ -320,7 +426,7 @@ class A2ML(BaseA2ML):
             .. code-block:: python
 
                 ctx = Context()
-                model = A2ML(ctx).actuals('D881079E1ED14FB', filename=<path_to_file>/actuals.csv)
+                A2ML(ctx).actuals('D881079E1ED14FB', filename=<path_to_file>/actuals.csv)
 
             .. code-block:: python
 
@@ -328,10 +434,19 @@ class A2ML(BaseA2ML):
                 actual_records = [['predicted_value_1', 'actual_value_1'], ['predicted_value_2', 'actual_value_2']]
                 columns = [target, 'actual']
 
-                model = A2ML(ctx).actuals('D881079E1ED14FB', data=actual_records,columns=columns)
+                A2ML(ctx).actuals('D881079E1ED14FB', data=actual_records,columns=columns)
+
+            .. code-block:: python
+
+                ctx = Context()
+                actual_records = [['predicted_value_1', 'actual_value_1'], ['predicted_value_2', 'actual_value_2']]
+                columns = [target, 'actual']
+
+                A2ML(ctx, "external").actuals('external_model_id', data=actual_records,columns=columns)
 
         """
-        return self.get_runner(locally, model_id, provider).execute_one_provider('actuals', model_id, filename, data, columns, actuals_at, actual_date_column, locally)
+        return self.get_runner(locally, model_id, provider).execute_one_provider('actuals', 
+            model_id, filename, data, columns, actuals_at, actual_date_column, experiment_params, locally)
 
 
     @show_result
@@ -372,17 +487,21 @@ class A2ML(BaseA2ML):
             locally(bool): Process review locally.
 
         Returns:
+            status(str): May be : started, error, completed, retrain
+            error(str): Description of error if status='error'
+            accuracy(float): Average accuracy of model(based on used metric) for review sensitivity period(see config.yml)
+
             ::
 
                 {
                     'result': True,
-                    'data': {'status': 'completed', 'error': ''}
+                    'data': {'status': 'completed', 'error': '', 'accuracy': 0.76}
                 }
 
         Examples:
             .. code-block:: python
 
                 ctx = Context()
-                model = A2ML(ctx).review(model_id='D881079E1ED14FB')
+                result = A2ML(ctx).review(model_id='D881079E1ED14FB')
         """
         return self.get_runner(locally, model_id, provider).execute_one_provider('review', model_id)
