@@ -25,7 +25,8 @@ class A2MLModel(BaseA2ML):
         self.local_runner = lambda: self.build_runner(ctx, provider, force_local=True)
 
     @show_result
-    def deploy(self, model_id, locally=False, review=True, provider=None):
+    def deploy(self, model_id, locally=False, review=False, provider=None,
+        name=None, algorithm=None, score=None, data_path=None, metadata=None):
         """Deploy a model locally or to specified provider(s).
 
         Args:
@@ -33,6 +34,11 @@ class A2MLModel(BaseA2ML):
             locally(bool): Deploys using a local model if True, on the Provider Cloud if False.
             review(bool): Should model support review based on actual data. The default is True.
             provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.
+            name (str): Friendly name for the model. Used as name for Review Endpoint
+            algorithm (str): Monitored model(external provider) algorithm name.
+            score (float): Monitored model(external provider) score.
+            data_path (str): Data path to fit model when deploy. Return new deployed model-id
+            metadata (dict): Additional parameter for the model. Used for accurcay report(report parameter)
 
         Returns:
             ::
@@ -46,13 +52,23 @@ class A2MLModel(BaseA2ML):
             .. code-block:: python
 
                 ctx = Context()
-                model = A2MLModel(ctx).deploy(model_id='D881079E1ED14FB', locally=True)
+                model = A2MLModel(ctx).deploy(model_id='D881079E1ED14FB', name='FirstExperiment')
+
+            .. code-block:: python
+
+                ctx = Context()
+                model = A2MLModel(ctx, 'external')
+                result = model.deploy(model_id=None, name="My external model.", algorithm='RandomForest', score=0.75)
+                model_id = result['data']['model_id']
+
         """
-        return self.get_runner(locally, model_id, provider).execute_one_provider('deploy', model_id, locally, review)
+        return self.get_runner(locally, model_id, provider).execute_one_provider('deploy',
+            model_id, locally, review, name, algorithm, score, data_path, metadata)
 
     @show_result
-    def predict(self, model_id, filename=None, data=None, columns=None, predicted_at=None, 
-            threshold=None, output=None, locally=False, provider=None):
+    def predict(self, model_id, filename=None, data=None, columns=None, predicted_at=None,
+            threshold=None, score=False, score_true_data=None,
+            output=None, no_features_in_result=None, locally=False, provider=None):
         """Predict results with new data against deployed model. Predictions are stored next to the file with data to be predicted on. The file name will be appended with suffix _predicted.
 
         Note:
@@ -61,13 +77,16 @@ class A2MLModel(BaseA2ML):
 
         Args:
             model_id(str): The deployed model id you want to use.
-            filename(str): The file with data to request predictions for.            
-            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFramme to_dict method
+            filename(str): The file with data to request predictions for.
+            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFrame to_dict('list') method
             columns(list): list of column names if data is array of records
             predicted_at: Predict data date. Use for review of historical data.
             threshold(float): For classification models only. This will return class probabilities with response.
+            score(bool): Calculate scores for predicted results.
+            score_true_data(str, pandas.DataFrame, dict): Data with true values to calculate scores. If missed, target from filename used for true values.            
             output(str): Output csv file path.
-            locally(bool): Predicts using a local model if True, on the Provider Cloud if False.
+            no_features_in_result(bool) : Do not return feature columns in prediction result. False by default
+            locally(bool, str): Predicts using a local model with auger.ai.predict if True, on the Provider Cloud if False. If set to "docker", then docker image used to run the model
             provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider set in costructor or config.
 
         Returns:
@@ -117,37 +136,64 @@ class A2MLModel(BaseA2ML):
                 # if rv[provider].result is True
                 # predictions are returned as rv[provider]['data']['predicted']
 
+            .. code-block:: python
+                
+                # Predict locally without config files. Model will automatically downloaded if not exists.
+                # To use local predict install a2ml[predict]
+                ctx = Context()
+                ctx.config.set('name', 'project name')
+                ctx.credentials = "Json string from a2ml ui settings"
+
+                rv = A2MLModel(ctx).predict(model_id, '../irises.csv',
+                    no_features_in_result = True, locally=True)
+                # if rv[provider].result is True
+                # predictions are stored in rv[provider]['data']['predicted']
+
         """
-        return self.get_runner(locally, model_id, provider).execute_one_provider('predict', filename, model_id, threshold, locally, data, columns, predicted_at, output)
+        return self.get_runner(locally, model_id, provider).execute_one_provider('predict', filename, model_id, 
+            threshold, locally, data, columns, predicted_at, output, no_features_in_result,
+            score, score_true_data )
 
     @show_result
-    def actuals(self, model_id, filename=None, data=None, columns=None, actuals_at=None, actual_date_column=None, locally=False, provider=None):
+    def actuals(self, model_id, filename=None, data=None, columns=None, actuals_at=None, 
+        actual_date_column=None, experiment_params=None, locally=False, provider=None):
         """Submits actual results(ground truths) for predictions of a deployed model. This is used to review and monitor active models.
 
         Note:
             It is assumed you have predictions against this model first.
 
             .. list-table:: actuals.csv
-                :widths: 50 50
+                :widths: 50 50 50
                 :header-rows: 1
 
-                * - target: predicted value
+                * - predicted ( or target): predicted value. If missed - predict called automatically
                   - actual
+                  - baseline_target: predicted value for baseline model (OPTIONAL)
                 * - Iris-setosa
                   - Iris-setosa
+                  - Iris-setosa
                 * - Iris-virginica
                   - Iris-virginica
-                * It may also contain train features to retrain while Review
+                  - Iris-virginica
+
+            It may also contain train features to predict(if target missed), retrain model while Review and for distribution chart
 
             This method support only one provider
 
         Args:
             model_id(str): The deployed model id you want to use.
             filename(str): The file with data to request predictions for.
-            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFramme to_dict method
+            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFrame to_dict('list') method
             columns(list): list of column names if data is array of records
             actuals_at: Actuals date. Use for review of historical data.
             actual_date_column(str): name of column in data which contains actual date
+            experiment_params(dict): parameters to calculate experiment metrics ::
+
+                start_date(date): experiment actuals start date
+                end_date(date):  experiment actuals end date
+                date_col(str): column name with date
+
+
             locally(bool): Process actuals locally.
             provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider set in costructor or config.
 
@@ -170,7 +216,7 @@ class A2MLModel(BaseA2ML):
             .. code-block:: python
 
                 ctx = Context()
-                model = A2MLModel(ctx).actuals('D881079E1ED14FB', filename=<path_to_file>/actuals.csv)
+                A2MLModel(ctx).actuals('D881079E1ED14FB', filename=<path_to_file>/actuals.csv)
 
             .. code-block:: python
 
@@ -178,13 +224,22 @@ class A2MLModel(BaseA2ML):
                 actual_records = [['predicted_value_1', 'actual_value_1'], ['predicted_value_2', 'actual_value_2']]
                 columns = [target, 'actual']
 
-                model = A2MLModel(ctx).actuals('D881079E1ED14FB', data=actual_records,columns=columns)
+                A2MLModel(ctx).actuals('D881079E1ED14FB', data=actual_records,columns=columns)
+
+            .. code-block:: python
+
+                ctx = Context()
+                actual_records = [['predicted_value_1', 'actual_value_1'], ['predicted_value_2', 'actual_value_2']]
+                columns = [target, 'actual']
+
+                A2MLModel(ctx, "external").actuals('external_model_id', data=actual_records,columns=columns)
 
         """
-        return self.get_runner(locally, model_id, provider).execute_one_provider('actuals', model_id, filename, data, columns, actuals_at, actual_date_column, locally)
+        return self.get_runner(locally, model_id, provider).execute_one_provider('actuals', 
+            model_id, filename, data, columns, actuals_at, actual_date_column, experiment_params, locally)
 
     @show_result
-    def review_alert(self, model_id, parameters = None, locally=False, provider=None):
+    def review_alert(self, model_id, parameters = None, locally=False, provider=None, name=None):
         """Update Review parameters.
 
         Args:
@@ -200,6 +255,12 @@ class A2MLModel(BaseA2ML):
 
                 * threshold (float)
                 * sensitivity (int): The amount of time(in hours) this metric must be at or below the threshold to trigger the alert.
+                * threshold_policy (all_values/average_value/any_value)
+
+                    - all_values: Default value. Trigger an alert when all values in sensitivity below threshold
+                    - average_value: Trigger an alert when average of values in sensitivity below threshold
+                    - any_value: Trigger an alert when any value in sensitivity below threshold
+
                 * action (no/retrain/retrain_deploy)
 
                     - no: no action should be executed
@@ -210,6 +271,7 @@ class A2MLModel(BaseA2ML):
 
             locally(bool): Process review locally.
             provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.
+            name (str): Friendly name for the model. Used as name for Review Endpoint
 
         Returns:
             ::
@@ -224,7 +286,7 @@ class A2MLModel(BaseA2ML):
                 ctx = Context()
                 model = A2MLModel(ctx).review_alert(model_id='D881079E1ED14FB')
         """
-        return self.get_runner(locally, model_id, provider).execute_one_provider('review_alert', model_id, parameters)
+        return self.get_runner(locally, model_id, provider).execute_one_provider('review_alert', model_id, parameters, name)
 
     @show_result
     def review(self, model_id, locally=False, provider=None):
@@ -236,18 +298,22 @@ class A2MLModel(BaseA2ML):
             provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.
 
         Returns:
+            status(str): May be : started, error, completed, retrain
+            error(str): Description of error if status='error'
+            accuracy(float): Average accuracy of model(based on used metric) for review sensitivity period(see config.yml)
+
             ::
 
                 {
                     'result': True,
-                    'data': {'status': 'completed', 'error': ''}
+                    'data': {'status': 'completed', 'error': '', 'accuracy': 0.76}
                 }
 
         Examples:
             .. code-block:: python
 
                 ctx = Context()
-                model = A2MLModel(ctx).review(model_id='D881079E1ED14FB')
+                result = A2MLModel(ctx).review(model_id='D881079E1ED14FB')
         """
         return self.get_runner(locally, model_id, provider).execute_one_provider('review', model_id)
 
@@ -280,12 +346,12 @@ class A2MLModel(BaseA2ML):
     def delete_actuals(self, model_id, with_predictions=False, begin_date=None, end_date=None, locally=False, provider=None):
         """Delete files with actuals and predcitions locally or from specified provider(s).
 
-        Args:
-            model_id (str): Model ID to delete actuals and predictions.
+        Args:
+            model_id (str): Model ID to delete actuals and predictions.
             with_predictions(bool):
             begin_date: Date to begin delete operations
             end_date: Date to end delete operations
-            locally(bool): Delete files from local model if True, on the Provider Cloud if False. The default is False.
+            locally(bool): Delete files from local model if True, on the Provider Cloud if False. The default is False.
             provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.
 
         Returns:
@@ -303,3 +369,54 @@ class A2MLModel(BaseA2ML):
                 A2MLModel(ctx).delete_actuals(model_id='D881079E1ED14FB')
         """
         return self.get_runner(locally, model_id, provider).execute_one_provider('delete_actuals', model_id, with_predictions, begin_date, end_date, locally)
+
+    @show_result
+    def get_info(self, model_id, locally=False, provider=None):
+        """Get information about model
+
+        Args:
+            model_id (str): Model ID to delete actuals and predictions.
+            locally(bool): Get information from local model if True, on the Provider Cloud if False. The default is False.
+            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.
+
+        Returns:
+            ::
+
+                {
+                    'result': True,
+                    'data': {} #detailed model information
+                }
+
+        Examples:
+            .. code-block:: python
+
+                ctx = Context()
+                res = A2MLModel(ctx).get_info('D881079E1ED14FB')
+        """
+        return self.get_runner(locally, model_id, provider).execute_one_provider('get_info', model_id, locally)
+
+    @show_result
+    def update(self, model_id, metadata, locally=False, provider=None):
+        """Update model metadata
+
+        Args:
+            model_id (str): Model ID to delete actuals and predictions.
+            metadata (dict): Model metadata to update
+            locally(bool): Get information from local model if True, on the Provider Cloud if False. The default is False.
+            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.
+
+        Returns:
+            ::
+
+                {
+                    'result': True,
+                    'data': {}
+                }
+
+        Examples:
+            .. code-block:: python
+
+                ctx = Context()
+                res = A2MLModel(ctx).update('D881079E1ED14FB', {'report': {}})
+        """
+        return self.get_runner(locally, model_id, provider).execute_one_provider('update', model_id, metadata, locally)
