@@ -24,7 +24,10 @@ class AugerBaseApi(object):
         params = {} if params is None else params
         if self.parent_api:
             api_request_path = self.parent_api.api_request_path
-            params['%s_id' % api_request_path] = self.parent_api.oid
+            try:
+                params['%s_id' % api_request_path] = self.parent_api.oid
+            except: 
+                pass    
         if self.object_name:
             params['name'] = self.object_name
         return self.rest_api.request_list(
@@ -134,6 +137,7 @@ class AugerBaseApi(object):
         if has_return_object:
             if object_properties:
                 self.object_id = object_properties.get('id')
+                self.object_name = object_properties.get('name') #name can be changed by hub                
                 if progress:
                     self.wait_for_status(progress)
             return self.properties()
