@@ -71,6 +71,8 @@ class AugerExperimentApi(AugerBaseApi):
             options['oversampling'] = config.get('experiment/oversampling')
         if config.get('experiment/estimate_trial_time'):
             options['apply_estimate_trial_time'] = config.get('experiment/estimate_trial_time')
+        if config.get('experiment/apply_eval_set') is not None:
+            options['apply_eval_set'] = config.get('experiment/apply_eval_set')            
         if config.get('experiment/max_cores_per_trial'):
             options['cpu_per_mt_algorithm'] = config.get('experiment/max_cores_per_trial')
         if config.get('experiment/max_concurrent_trials'):
@@ -79,8 +81,35 @@ class AugerExperimentApi(AugerBaseApi):
             options["algorithms_to_exlude"] = config.get_list('experiment/blocked_models')
         if config.get('experiment/allowed_models'):
             options["allowed_algorithms"] = config.get_list('experiment/allowed_models')
+
+        if config.get('experiment/blocked_preprocessors'):
+            options["blocked_preprocessors"] = config.get_list('experiment/blocked_preprocessors')
+        if config.get('experiment/allowed_preprocessors'):
+            options["allowed_preprocessors"] = config.get_list('experiment/allowed_preprocessors')
+        if config.get('experiment/preprocessors'):
+            options["preprocessors"] = config.get_list('experiment/preprocessors')
+
         if config.get('experiment/exit_score'):
             options['exit_score'] = config.get('experiment/exit_score')
+        if config.get('experiment/score_top_count'):
+            options['score_top_count'] = config.get('experiment/score_top_count')
+        if config.get('experiment/build_metrics_in_trial', None) is not None:
+            options['build_metrics_in_trial'] = config.get('experiment/build_metrics_in_trial')
+        if config.get('experiment/optimizer_batch_size', None) is not None:
+            options['optimizer_batch_size'] = config.get('experiment/optimizer_batch_size')
+        if config.get('experiment/cpu_per_mt_algorithm', None) is not None:
+            options['cpu_per_mt_algorithm'] = config.get('experiment/cpu_per_mt_algorithm')
+        if config.get('experiment/optimizers_names', None) is not None:
+            options['optimizers_names'] = config.get('experiment/optimizers_names')
+        if config.get('experiment/optimizers', None) is not None:
+            options['optimizers'] = config.get('experiment/optimizers')
+        if config.get('experiment/search_space', None) is not None:
+            options['search_space'] = config.get('experiment/search_space')
+
+        if config.get('review/alert/retrain_policy/type'):
+            options['retrain_policy_type'] = config.get('review/alert/retrain_policy/type')
+        if config.get('review/alert/retrain_policy/value'):
+            options['retrain_policy_value'] = config.get('review/alert/retrain_policy/value')
 
         split_options = {}
         if config.get('experiment/validation_size'):
@@ -89,10 +118,21 @@ class AugerExperimentApi(AugerBaseApi):
             split_options['shuffleData'] = config.get('experiment/shuffle')
         if config.get('experiment/timeseries_fold_split') is not None:
             split_options['timeseriesSplit'] = config.get('experiment/timeseries_fold_split')
+        if config.get('experiment/test_size_limit') is not None:
+            split_options['test_size_limit'] = config.get('experiment/test_size_limit')
+        if config.get('experiment/groups_col') is not None:
+            split_options['groups_col'] = config.get('experiment/groups_col')
 
         if split_options:
             options['splitOptions'] = split_options
 
+        if  config.get('review/roi/filter'):
+            options['roi_metric'] = {
+                'filter': str(config.get('review/roi/filter', '')), 
+                'revenue': str(config.get('review/roi/revenue')), 
+                'investment': str(config.get('review/roi/investment'))
+            }
+
         return options
             
     def get_experiment_settings(self):
