@@ -64,22 +64,42 @@ class AugerExperimentSessionApi(AugerBaseApi):
         leaderboard, score_name = [], None
         for item in iter(trial_api.list()):
             score_name = item.get('score_name')
-            leaderboard.append({
+            l_item = {
                 'model id': item.get('id'),
                 'algorithm': item.get('hyperparameter').\
                     get('algorithm_name').split('.')[-1],
                 item.get('score_name'):\
                     '{0:.4f}'.format(item.get('score_value'))
-            })
+            }
+            review_metric = self.ctx.config.get('review/metric')
+            if review_metric == 'roi':
+                review_metric = 'mda'
+            if review_metric:
+                l_item[review_metric] = \
+                    '{0:.4f}'.format(item.get('raw_data', {}).get('all_scores', {}).get(review_metric, 0.0))
+
+            l_item['score'] = float(item.get('score_value'))
+            l_item['fold_scores'] = []
+            for item in item.get('raw_data', {}).get('fold_scores', []):
+                l_item['fold_scores'].append('{0:.4f}'.format(item))
+
+            leaderboard.append(l_item)
 
         if score_name:
-            leaderboard.sort(key=lambda t: list(t.values())[-1], reverse=False)
-            
+            leaderboard.sort(key=lambda t: t['score'], reverse=False)
+        
+        for item in leaderboard:
+            del item['score']
+                
         return leaderboard
 
     def update_settings(self):
         from .experiment import AugerExperimentApi
 
+        if not self.ctx.config.get('experiment'):
+            self.ctx.log("Config does not contain experiment section. Do not update Review retrain options")
+            return
+
         session_props = self.properties()
         props = session_props.get('model_settings',{}).get('evaluation_options')
         config_props = AugerExperimentApi.get_experiment_options(self.ctx.config)
