@@ -14,7 +14,8 @@ class AugerPredictionApi(AugerBaseApi):
             self.parent_id_name = "endpoint_id"
             self._set_api_request_path("AugerEndpointPredictionApi")
 
-    def create(self, records, features, threshold=None, file_url=None, predicted_at=None):
+    def create(self, records, features, threshold=None, file_url=None, predicted_at=None, 
+        no_features_in_result=None, score=False, score_true_data=None):
         params = {
             'records': records, 
             'features': features,
@@ -33,4 +34,13 @@ class AugerPredictionApi(AugerBaseApi):
         if predicted_at:
             params['predicted_at'] = str(predicted_at)
 
+        if no_features_in_result is not None:
+            params['no_features_in_result'] = no_features_in_result
+
+        if score:
+            params['score'] = score
+
+        if score_true_data:
+            params['score_true_data'] = score_true_data
+
         return self._call_create(params, ['requested', 'running'])
