@@ -26,10 +26,17 @@ class RestApi(object):
             oid = params['id']
             del params['id']
             #print(method, oid, params)
-            return getattr(self.hub_client, method)(oid, **params)
+            res = getattr(self.hub_client, method)(oid, **params)
         else:
-            #print(method, params)
-            return getattr(self.hub_client, method)(**params)
+            # if method == 'create_endpoint_prediction' or method == 'create_endpoint_actual':
+            #     print(method, params.keys())
+            # else:
+            #     print(method, params)
+
+            res = getattr(self.hub_client, method)(**params)
+
+        #print(res)    
+        return res
 
     def call(self, method, params={}):
         result = self.call_ex(method, params)
