@@ -1,7 +1,6 @@
 import os
 import shutil
 import subprocess
-from zipfile import ZipFile
 import sys
 import pandas as pd
 
@@ -9,10 +8,11 @@ from .deploy import ModelDeploy
 from ..cloud.cluster import AugerClusterApi
 from ..cloud.pipeline import AugerPipelineApi
 from ..exceptions import AugerException
-from a2ml.api.utils import fsclient
+from a2ml.api.utils import fsclient, getsizeof_deep
 from a2ml.api.utils.dataframe import DataFrame
 from a2ml.api.model_review.model_helper import ModelHelper
 from ..decorators import with_project
+from ..project import Project
 
 
 class ModelPredict():
@@ -22,16 +22,25 @@ class ModelPredict():
         super(ModelPredict, self).__init__()
         self.ctx = ctx
 
-    def execute(self, filename, model_id, threshold=None, locally=False, data=None, columns=None, predicted_at=None, output=None):
-        if filename and not (filename.startswith("http:") or filename.startswith("https:")) and\
+    def execute(self, filename, model_id, threshold=None, locally=False, data=None, columns=None, 
+            predicted_at=None, output=None, no_features_in_result=None,
+            score=False, score_true_data=None):
+        if filename is not None and isinstance(filename, str) and \
+            not (filename.startswith("http:") or filename.startswith("https:")) and \
             not fsclient.is_s3_path(filename):
             self.ctx.log('Predicting on data in %s' % filename)
             filename = os.path.abspath(filename)
 
         if locally:
-            predicted = self._predict_locally(filename, model_id, threshold, data, columns, predicted_at, output)
+            if locally == "docker":
+                predicted = self._predict_locally_in_docker(filename, model_id, threshold, data, columns, predicted_at, output, 
+                    no_features_in_result, score, score_true_data)
+            else:
+                predicted = self._predict_locally(filename, model_id, threshold, data, columns, predicted_at, output, 
+                    no_features_in_result, score, score_true_data)
         else:
-            predicted = self._predict_on_cloud(filename, model_id, threshold, data, columns, predicted_at, output)
+            predicted = self._predict_on_cloud(filename, model_id, threshold, data, columns, predicted_at, output, 
+                no_features_in_result, score, score_true_data)
 
         return predicted
 
@@ -48,6 +57,7 @@ class ModelPredict():
         records = None
         features = None
         max_records_size = 10*1024
+
         if filename:
             if not (filename.startswith("http:") or filename.startswith("https:")) and \
                not fsclient.is_s3_path(filename):
@@ -63,10 +73,12 @@ class ModelPredict():
         elif data is not None and isinstance(data, pd.DataFrame):
             if len(data) < 100:
                 send_records = True
-        elif sys.getsizeof(data) > max_records_size:
-            self.ctx.log("Size of data %s KB > %s KB, so send file to hub." % (sys.getsizeof(data)/1024, max_records_size/1024))
         else:
-            send_records = True
+            size = getsizeof_deep(data)
+            if size is None or size > max_records_size:
+                self.ctx.log("Size of data %.2f KB > %.2f KB, so send file to hub." % (size/1024, max_records_size/1024))
+            else:
+                send_records = True
 
         if send_records:
             ds = DataFrame.create_dataframe(filename, data, columns)
@@ -84,7 +96,15 @@ class ModelPredict():
 
         return records, features, file_url, data is not None and isinstance(data, pd.DataFrame)
 
-    def _predict_on_cloud(self, filename, model_id, threshold, data, columns, predicted_at, output):
+    def _check_model_project(self, pipeline_api):
+        model_project_name = Project(self.ctx, project_id=pipeline_api.properties().get('project_id')).properties().get('name')
+
+        if model_project_name != self.ctx.config.get('name'):
+            raise AugerException("Project name: %s in config.yml is different from model project name: %s. Please change name in config.yml."%(
+                self.ctx.config.get('name'), model_project_name))
+
+    def _predict_on_cloud(self, filename, model_id, threshold, data, columns, predicted_at, 
+        output, no_features_in_result, score, score_true_data):
         records, features, file_url, is_pandas_df = self._process_input(filename, data, columns)
         temp_file = None
         ds_result = None
@@ -92,10 +112,19 @@ class ModelPredict():
             ds_result =  DataFrame.create_dataframe(None, [], features+[self.ctx.config.get('target')])
         else:
             pipeline_api = AugerPipelineApi(self.ctx, None, model_id)
-            predictions = pipeline_api.predict(records, features, threshold=threshold, file_url=file_url, predicted_at=predicted_at)
+            predictions = pipeline_api.predict(records, features, threshold=threshold, file_url=file_url, 
+                predicted_at=predicted_at, no_features_in_result=no_features_in_result,
+                score=score, score_true_data=score_true_data)
+
+            try:
+                ds_result = DataFrame.create_dataframe(predictions.get('signed_prediction_url'),
+                    records=predictions.get('data'), features=predictions.get('columns'))
+            except Exception as e:
+                self._check_model_project(pipeline_api)
+
+                msg = "Prediction result file(%s) cannot be downloaded."%predictions.get('signed_prediction_url')
+                raise AugerException(msg+"Please contact support.")
 
-            ds_result = DataFrame.create_dataframe(predictions.get('signed_prediction_url'),
-                records=predictions.get('data'), features=predictions.get('columns'))
             temp_file = ds_result.options['data_path'] if predictions.get('signed_prediction_url') else None
 
         try:
@@ -103,11 +132,10 @@ class ModelPredict():
             ds_result.loaded_columns = columns
             ds_result.from_pandas = is_pandas_df
 
-            is_model_loaded, model_path_1, model_name = \
-                ModelDeploy(self.ctx, None).verify_local_model(model_id)
-            model_path = None
-            if is_model_loaded:
-                model_path = os.path.join(model_path_1, "model-%s"%model_id, 'model')
+            # Save prediction in local model folder if exist
+            is_model_loaded, model_path = ModelDeploy(self.ctx, None).verify_local_model(model_id)
+            if not is_model_loaded:
+                model_path = None
 
             return ModelHelper.save_prediction(ds_result,
                 prediction_id = None, json_result=False, count_in_result=False, prediction_date=predicted_at,
@@ -116,17 +144,60 @@ class ModelPredict():
             if temp_file:
                 fsclient.remove_file(temp_file)
 
-    def _predict_locally(self, filename_arg, model_id, threshold, data, columns, predicted_at, output):
-        model_deploy = ModelDeploy(self.ctx, None)
-        is_model_loaded, model_path, model_name = \
-            model_deploy.verify_local_model(model_id)
+    def _predict_locally(self, filename_arg, model_id, threshold, data, columns, predicted_at, 
+        output, no_features_in_result, score, score_true_data):
+        from auger_ml.model_exporter import ModelExporter
 
+        is_model_loaded, model_path = ModelDeploy(self.ctx, None).verify_local_model(model_id)
         if not is_model_loaded:
             raise AugerException('Model isn\'t loaded locally. '
                 'Please use a2ml deploy command to download model.')
 
-        model_path, model_existed = self._extract_model(model_name)
-        model_options = fsclient.read_json_file(os.path.join(model_path, "model", "options.json"))
+        if columns is not None:
+            columns = list(columns)
+
+        if filename_arg is None and data is not None and isinstance(data, pd.DataFrame):
+            filename_arg = data
+
+        if score and score_true_data is None:
+            options = fsclient.read_json_file(os.path.join(model_path, "options.json"))            
+            ds = DataFrame.create_dataframe(filename_arg, data, [options['targetFeature']])
+            score_true_data = ds.df
+                
+        res, options = ModelExporter({}).predict_by_model_to_ds(model_path, 
+            path_to_predict=filename_arg, records=data, features=columns, 
+            threshold=threshold, no_features_in_result=no_features_in_result)
+
+        ds_result = DataFrame({'data_path': None})
+        ds_result.df = res.df
+        ds_result.loaded_columns = columns
+        if isinstance(data, pd.DataFrame):
+            ds_result.from_pandas = True
+
+        predictions = ModelHelper.save_prediction(ds_result,
+            prediction_id = None, json_result=False, count_in_result=False, prediction_date=predicted_at,
+            model_path=model_path, model_id=model_id, output=output)
+
+        if not score:    
+            return predictions
+
+        scores = ModelExporter({}).score_by_model(model_path, predictions=predictions, 
+            test_path = score_true_data)
+                
+        return {'predicted': predictions, 'scores': scores}
+
+        # return ModelExporter({}).predict_by_model(model_path=model_path,
+        #     path_to_predict=filename_arg, records=data, features=columns, 
+        #     threshold=threshold, prediction_date=predicted_at, 
+        #     no_features_in_result=no_features_in_result) #, output=output)
+
+    def _predict_locally_in_docker(self, filename_arg, model_id, threshold, data, columns, predicted_at, 
+        output, no_features_in_result, score, score_true_data):
+        model_deploy = ModelDeploy(self.ctx, None)
+        is_model_loaded, model_path = model_deploy.verify_local_model(model_id, add_model_folder=False)
+        if not is_model_loaded:
+            raise AugerException('Model isn\'t loaded locally. '
+                'Please use a2ml deploy command to download model.')
 
         filename = filename_arg
         if not filename:
@@ -134,15 +205,7 @@ class ModelPredict():
             filename = os.path.join(self.ctx.config.get_path(), '.augerml', 'predict_data.csv')
             ds.saveToCsvFile(filename, compression=None)
 
-        try:
-            predicted = \
-                self._docker_run_predict(filename, threshold, model_path)
-        finally:
-            # clean up unzipped model
-            # if it wasn't unzipped before
-            if not model_existed:
-                fsclient.remove_folder(model_path)
-                model_path = None
+        predicted = self._docker_run_predict(filename, threshold, model_path, score, score_true_data)
 
         if not filename_arg:
             ds_result = DataFrame.create_dataframe(predicted)
@@ -160,23 +223,14 @@ class ModelPredict():
 
         return predicted
 
-    def _extract_model(self, model_name):
-        model_path = os.path.splitext(model_name)[0]
-        model_existed = os.path.exists(model_path)
-
-        if not model_existed:
-            with ZipFile(model_name, 'r') as zip_file:
-                zip_file.extractall(model_path)
-
-        return model_path, model_existed
-
-    def _docker_run_predict(self, filename, threshold, model_path):
+    def _docker_run_predict(self, filename, threshold, model_path, score, score_true_data):
         cluster_settings = AugerClusterApi.get_cluster_settings(self.ctx)
         docker_tag = cluster_settings.get('kubernetes_stack')
         predict_file = os.path.basename(filename)
         data_path = os.path.abspath(os.path.dirname(filename))
         model_path = os.path.abspath(model_path)
 
+        #TODO: support score, score_true_data
         call_args = "--verbose=True --path_to_predict=./model_data/%s %s" % \
             (predict_file, "--threshold=%s" % str(threshold) if threshold else '')
 
