@@ -7,8 +7,8 @@ from ..credentials import Credentials
 class Project(AugerProjectApi):
     """Auger Cloud Projects(s) management"""
 
-    def __init__(self, ctx, project_name=None):
+    def __init__(self, ctx, project_name=None, project_id=None):
         credentials = Credentials(ctx).load()
 
         org = AugerOrganizationApi(ctx, credentials.organization)
-        super(Project, self).__init__(ctx, org, project_name)
+        super(Project, self).__init__(ctx, org, project_name, project_id)
