@@ -15,27 +15,36 @@ class AugerModel(object):
     @error_handler
     @authenticated
     @with_project(autocreate=False)
-    def deploy(self, project, model_id, locally, review):
-        model_id = Model(self.ctx, project).deploy(model_id, locally, review)
+    def deploy(self, project, model_id, locally, review, name, algorithm, score, data_path, metadata=None):
+        model_id = Model(self.ctx, project).deploy(model_id, locally, review, name, algorithm, score, data_path, metadata)
         return {'model_id': model_id}
 
     @error_handler
     @authenticated
-    @with_project(autocreate=False)
-    def predict(self, project, filename, model_id, threshold, locally, data, columns, predicted_at, output):
-        predicted = Model(self.ctx, project).predict(
-            filename, model_id, threshold, locally, data, columns, predicted_at, output)
+    #@with_project(autocreate=False)
+    def predict(self, filename, model_id, threshold, locally, data, columns, predicted_at, output, 
+        no_features_in_result, score, score_true_data):
+        if locally:
+            self.deploy(model_id, locally, review=False, name=None, algorithm=None, score=None, data_path=None)
+
+        predicted = Model(self.ctx, project=None).predict(
+            filename, model_id, threshold, locally, data, columns, predicted_at, output, 
+            no_features_in_result, score, score_true_data)
 
         if filename:
             self.ctx.log('Predictions stored in %s' % predicted)
 
+        if isinstance(predicted, dict) and 'predicted' in predicted:
+            return predicted
+
         return {'predicted': predicted}
 
     @error_handler
     @authenticated
     @with_project(autocreate=False)
-    def actuals(self, project, model_id, filename=None, data=None, columns=None, actuals_at=None, actual_date_column=None, locally=False):
-        return Model(self.ctx, project).actuals(model_id, filename, data, columns, actuals_at, actual_date_column, locally)
+    def actuals(self, project, model_id, filename=None, data=None, columns=None, actuals_at=None, 
+        actual_date_column=None, experiment_params=None, locally=False):
+        return Model(self.ctx, project).actuals(model_id, filename, data, columns, actuals_at, actual_date_column, experiment_params, locally)
 
     @error_handler
     @authenticated
@@ -46,8 +55,8 @@ class AugerModel(object):
     @error_handler
     @authenticated
     @with_project(autocreate=False)
-    def review_alert(self, project, model_id, parameters):
-        return Model(self.ctx, project).review_alert(model_id, parameters)
+    def review_alert(self, project, model_id, parameters, name):
+        return Model(self.ctx, project).review_alert(model_id, parameters, name)
 
     @error_handler
     @authenticated
@@ -68,3 +77,15 @@ class AugerModel(object):
         Model(self.ctx, project).undeploy(model_id, locally)
         return {'model_id': model_id}
 
+
+    @error_handler
+    @authenticated
+    #@with_project(autocreate=False)
+    def get_info(self, model_id, locally):
+        return Model(self.ctx, project=None).get_info(model_id, locally)
+
+    @error_handler
+    @authenticated
+    #@with_project(autocreate=False)
+    def update(self, model_id, metadata, locally):
+        return Model(self.ctx, project=None).update(model_id, metadata, locally)
