@@ -20,7 +20,7 @@ class AzureModel(object):
 
     @error_handler
     @authenticated
-    def deploy(self, model_id, locally, review):
+    def deploy(self, model_id, locally, review, name=None, algorithm=None, score=None, data_path=None):
         if locally:
             is_loaded, model_path = self.verify_local_model(model_id)
             if is_loaded:
@@ -34,6 +34,7 @@ class AzureModel(object):
 
         result = self._deploy_locally(model_id, model_run, ws, experiment) if locally else \
             self._deploy_remotly(model_id, model_run, ws, experiment)
+        result = True
 
         model_features, target_categories = self._get_remote_model_features(model_run)
         feature_importance = self._get_feature_importance(model_run)
@@ -238,8 +239,8 @@ def get_df(data):
     @error_handler
     @authenticated
     def predict(self, filename, model_id, threshold=None, locally=False, data=None, columns=None,
-        predicted_at=None, output=None, json_result=False, count_in_result=False, prediction_id=None
-        ):
+        predicted_at=None, output=None, json_result=False, count_in_result=False, prediction_id=None,
+        no_features_in_result=None):
         ds = DataFrame.create_dataframe(filename, data, columns)
         model_path = self.ctx.config.get_model_path(model_id)
         options = fsclient.read_json_file(os.path.join(model_path, "options.json"))
@@ -399,7 +400,7 @@ def get_df(data):
         return model_features, target_categories
 
     def _get_feature_importance(self, model_run):
-        from azureml.explain.model._internal.explanation_client import ExplanationClient
+        from azureml.interpret._internal.explanation_client import ExplanationClient
 
         try:
             client = ExplanationClient.from_run(model_run)
@@ -552,7 +553,7 @@ def get_df(data):
 
     @error_handler
     @authenticated
-    def review_alert(self, model_id, parameters):
+    def review_alert(self, model_id, parameters, name):
         raise AzureException("Not Implemented. Set use_auger_cloud: True in config.yml")
 
     @error_handler
