@@ -4,9 +4,9 @@ import logging
 import numpy as np
 import json
 
-from a2ml.api.utils import get_uid, get_uid4, fsclient, remove_dups_from_list
+from a2ml.api.utils import get_uid, get_uid4, fsclient, remove_dups_from_list, sort_arrays
 from a2ml.api.utils.dataframe import DataFrame
-
+from a2ml.api.roi.calculator import Calculator as RoiCalculator
 
 class ModelHelper(object):
 
@@ -100,7 +100,7 @@ class ModelHelper(object):
 
     # @staticmethod
     # def _get_score_byname(scoring):
-    #     from sklearn.metrics.scorer import get_scorer
+    #     from sklearn.metrics import get_scorer
     #     from sklearn.metrics import SCORERS
 
     #     #TODO: below metrics does not directly map to sklearn:
@@ -157,10 +157,12 @@ class ModelHelper(object):
 
     @staticmethod
     def calculate_scores(options, y_test, X_test=None, estimator=None, y_pred=None, raise_main_score=True):
-        from sklearn.metrics.scorer import get_scorer
+        from sklearn.metrics import get_scorer
         from sklearn.model_selection._validation import _score
+        from sklearn.metrics import confusion_matrix
+
         # For calculate_scores
-        from .scores.regression import spearman_correlation_score
+        from .scores.regression import spearman_correlation_score, mae_ex
         from .scores.classification import AUC_weighted_score
 
         import inspect
@@ -182,8 +184,38 @@ class ModelHelper(object):
             else:
                 logging.error("calculate_scores: no scaling found for target fold group: %s"%options['fold_group'])
 
+        if options.get("score_top_count"):
+            if y_pred is None:
+                y_pred = estimator.predict(X_test)
+
+            y_pred, y_test = sort_arrays(y_pred, y_test, options.get("score_top_count"))
+            
         all_scores = {}
+        if y_pred is not None:
+            if options.get('binaryClassification'):
+                res = confusion_matrix(y_test, y_pred).ravel()    
+                #tn, fp, fn, tp
+                all_scores['TN'] = 0
+                all_scores['FP'] = 0
+                all_scores['FN'] = 0
+                all_scores['TP'] = 0
+
+                if len(res) > 0:
+                    all_scores['TN'] = res[0]
+                if len(res) > 1:
+                    all_scores['FP'] = res[1]
+                if len(res) > 2:
+                    all_scores['FN'] = res[2]
+                if len(res) > 3:
+                    all_scores['TP'] = res[3]
+            elif options.get("task_type") == "regression":
+                all_scores['mae_over'], all_scores['mae_under'] = \
+                    mae_ex(y_test, y_pred)
+
         for scoring in options.get('scoreNames', []):
+            if scoring.upper() in ['TN', 'FP', 'FN', 'TP']:
+                continue
+
             try:
                 if options.get('task_type') == "timeseries":
                     # from auger_ml.preprocessors.space import ppspace_is_timeseries_model
@@ -211,11 +243,15 @@ class ModelHelper(object):
                     #all_scores['scoring'] = scorer(estimator, X_test, y_test)
 
 
-                if np.isnan(all_scores[scoring]):
+                if not isinstance(all_scores[scoring], list) and np.isnan(all_scores[scoring]):
                     all_scores[scoring] = 0
 
             except Exception as e:
-                #logging.exception("Score failed.")
+                # import traceback
+                # print("Score %s for algorithm %s failed to build: %s" % (
+                #     scoring, options.get('algorithm_name'), str(e)))
+                # print(traceback.format_exc())
+
                 if scoring == options.get('scoring', None) and raise_main_score:
                     raise
 
@@ -411,5 +447,78 @@ class ModelHelper(object):
         if options.get('timeSeriesFeatures'):
             selected_cols.extend(options.get('timeSeriesFeatures'))
 
+        selected_cols.extend(ModelHelper.get_roi_features(options))
         return remove_dups_from_list(selected_cols)
 
+    @staticmethod
+    def get_roi_features(options):
+        if not options.get('roi_metric'):
+            return []
+
+        roi_features = RoiCalculator(
+            filter = options['roi_metric'].get('filter'),
+            revenue = options['roi_metric'].get('revenue'),
+            investment = options['roi_metric'].get('investment')
+        ).get_var_names()
+        logging.info("ROI features from roi metric: %s"%roi_features)
+
+        return roi_features
+
+    @staticmethod
+    def create_model_options_file(options_path, scoring, target_column, task_type, binary_classification):
+        options = {}
+        options["targetFeature"] = target_column
+        options["task_type"] = task_type
+        options["scoring"] = scoring
+        options["score_name"] = scoring
+
+        if task_type == "classification":
+            options["classification"] = True
+
+        options['binaryClassification'] = True if binary_classification else False
+        options['external_model'] = True
+
+        options["scoreNames"] = ModelHelper.get_available_scores(options)
+
+        fsclient.write_json_file(options_path, options)
+
+        return options
+
+    @staticmethod    
+    def get_available_scores(options):
+        ADD_SCORES = ['f1', 'precision', 'recall']
+        SUFFIXES = ['_micro', '_macro', '_weighted']  # '_samples'
+
+        if options.get('classification', True):
+            add_scores = []
+            for item in ADD_SCORES:
+                add_scores.extend([item + s for s in SUFFIXES])
+
+            if options.get('binaryClassification', False):
+                advanced_binary = ['cohen_kappa_score', 'matthews_corrcoef']    # 'fowlkes_mallows_score'
+                return ['accuracy', 'average_precision', 'neg_log_loss', 'roc_auc', 'gini'] \
+                    + ADD_SCORES + add_scores + advanced_binary
+            else:
+                # jaccard_similarity_score
+                return ['accuracy', 'neg_log_loss'] + add_scores
+        else:
+            return ['explained_variance', 'neg_median_absolute_error', 'neg_mean_absolute_error',
+                    'neg_mean_squared_error', 'neg_mean_squared_log_error', 'r2', 'neg_rmsle',
+                    'neg_mase', 'neg_mape', 'mda', 'neg_rmse']
+
+    def update_model_options_file(options_path, options, ds_actuals):
+        ds_actuals.options = options
+
+        feature_columns = ds_actuals.columns
+        if "targetFeature" in feature_columns:
+            feature_columns.remove(options["targetFeature"])
+        ds_actuals.options["featureColumns"] = feature_columns
+        ds_actuals.options["originalFeatureColumns"] = feature_columns
+
+        summary = ds_actuals.getSummary()
+        binaryClassification = ds_actuals.options.get('binaryClassification', False)
+        options = ds_actuals.update_options_by_dataset_statistics(summary["stat_data"])
+        options['binaryClassification'] = binaryClassification
+        fsclient.write_json_file(options_path, options)
+
+        return options
