@@ -59,6 +59,17 @@ def url_encode(path):
 
     return quote(path, safe='#&%:/?*=\'')
 
+def parse_url(remote_path):
+    try:
+        from urllib.parse import urlparse, parse_qs, quote
+    except ImportError:
+        from urlparse import urlparse, parse_qs, quote
+
+    uri = urlparse(remote_path)
+    params = parse_qs(uri.query)
+
+    return uri.path, params
+
 def get_remote_file_info(remote_path):
     from urllib.request import urlopen
     import urllib
@@ -204,14 +215,14 @@ def convert_to_date(date):
     else:
         return date
 
-@contextlib.contextmanager
-def convert_source(source, name):
-    if source is not None and isinstance(source, pd.DataFrame):
-        with fsclient.save_atomic("%s.parquet"%name, move_file=False) as local_path:
-            source.to_parquet(local_path, index=False, compression="gzip")
-            yield local_path
-    else:
-        yield source
+# @contextlib.contextmanager
+# def convert_source(source, name):
+#     if source is not None and isinstance(source, pd.DataFrame):
+#         with fsclient.save_atomic("%s.parquet"%name, move_file=False) as local_path:
+#             source.to_parquet(local_path, index=False, compression="gzip")
+#             yield local_path
+#     else:
+#         yield source
 
 def retry_helper(func, retry_errors=[], num_try=10, delay=10, ctx=None):
     nTry = 0
@@ -238,4 +249,43 @@ def retry_helper(func, retry_errors=[], num_try=10, delay=10, ctx=None):
                 time.sleep(delay*nTry)
             else:
                 raise                
-                
+
+def sort_arrays(ar1, ar2, top_n=None, desc=True):
+    p = ar1.argsort()
+    if desc:
+        p = p[::-1]
+    if top_n:
+        p = p[:top_n]
+
+    return ar1[p], ar2[p]
+
+def getsizeof_deep(obj):
+    import gc
+    import sys
+
+    sz = 0
+    try:
+        marked = {id(obj)}
+        obj_q = [obj]
+
+        while obj_q:
+            sz += sum(map(sys.getsizeof, obj_q))
+
+            # Lookup all the object referred to by the object in obj_q.
+            # See: https://docs.python.org/3.7/library/gc.html#gc.get_referents
+            all_refr = ((id(o), o) for o in gc.get_referents(*obj_q))
+
+            # Filter object that are already marked.
+            # Using dict notation will prevent repeated objects.
+            new_refr = {o_id: o for o_id, o in all_refr if o_id not in marked and not isinstance(o, type)}
+
+            # The new obj_q will be the ones that were not marked,
+            # and we will update marked with their ids so we will
+            # not traverse them again.
+            obj_q = new_refr.values()
+            marked.update(new_refr.keys())
+    except:
+        logging.exception("getsizeof_deep failed.")
+        sz = None
+
+    return sz
