@@ -19,6 +19,18 @@ def _get_fsclient_bypath(path):
     else:
         return LocalFSClient()
 
+def get_file_base_name(file_name):
+    if '.' in file_name:
+        separator_index = file_name.index('.')
+        base_name = file_name[:separator_index]
+        return base_name
+    else:
+        return file_name
+
+def get_path_base_name(path):
+    file_name = os.path.basename(path)
+    return get_file_base_name(file_name)
+
 
 def create_folder(path):
     client = _get_fsclient_bypath(path)
