@@ -1,39 +1,39 @@
+import boto3
 import botocore
-import os
 import datetime
-from dateutil.tz import tzutc
-import mimetypes
+import json
 import logging
+import mimetypes
+import os
 import time
-from a2ml.api.utils import retry_helper
+import uuid
 
+from a2ml.api.utils import retry_helper
+from dateutil.tz import tzutc
+from urllib.parse import urlparse
+
+AWS_S3_HOST = "s3.amazonaws.com"
 
 def retry_handler(decorated):
     def wrapper(self, *args, **kwargs):
-        return retry_helper(lambda: decorated(self, *args, **kwargs), ['InvalidAccessKeyId'])
+        return retry_helper(lambda: decorated(self, *args, **kwargs), ['InvalidAccessKeyId', 'NoSuchKey', 'Please try again'])
 
     return wrapper
 
 class BotoClient:
-    def __init__(self, region=None):
-        import boto3
+    def __init__(self, region=None, aws_role_arn=None, endpoint_url=None):
+        self.endpoint_url = endpoint_url or os.environ.get('S3_ENDPOINT_URL')
+        self.aws_role_arn = aws_role_arn or os.environ.get('AWS_ROLE_ARN')
+        self.region = region
+        self.client = self._build_client('s3')
 
-        endpoint_url = os.environ.get('S3_ENDPOINT_URL')
-
-        self.client = None
-        if endpoint_url:
-            self.client = boto3.client(
-                's3',
-                endpoint_url=endpoint_url,
-                config=boto3.session.Config(signature_version='s3v4'),
-                region_name=region,
-            )
-        else:
-            self.client = boto3.client(
-                's3',
-                config=boto3.session.Config(signature_version='s3v4'),
-                region_name=region,
-            )
+    def _build_client(self, service_name):
+        return boto3.client(
+            service_name,
+            endpoint_url=self.endpoint_url,
+            config=boto3.session.Config(signature_version='s3v4'),
+            region_name=self.region,
+        )
 
     @retry_handler
     def get_waiter(self, *args, **kwargs):
@@ -110,23 +110,102 @@ class BotoClient:
         return self.client.copy(*args, **kwargs)
 
     @retry_handler
-    def generate_presigned_url_ex(self, bucket, path):
-        import boto3
+    def put_bucket_cors(self, *args, **kwargs):
+        return self.client.put_bucket_cors(*args, **kwargs)
+
+    @retry_handler
+    def generate_presigned_url_ex(self, bucket, key, method="GET", expires_in=None, max_content_length=None):
+        response = self.client.get_bucket_location(Bucket=bucket)
 
-        response = self.client.get_bucket_location(
-            Bucket=bucket
-        )
         s3_client = boto3.client(
             's3',
-            config=boto3.session.Config(signature_version='s3v4',
-                region_name=response.get('LocationConstraint'))
+            endpoint_url=self.endpoint_url,
+            config=boto3.session.Config(
+                signature_version='s3v4',
+                region_name=response.get('LocationConstraint'),
+                s3={'addressing_style': 'virtual'},
+            )
         )
 
-        return s3_client.generate_presigned_url(
-            ClientMethod='get_object',
-            Params={
-                'Bucket': bucket,
-                'Key': path
+        if method == 'POST':
+            conditions = [{"success_action_status": "200"}]
+
+            if max_content_length:
+                conditions.append(["content-length-range", 0, max_content_length])
+
+            return s3_client.generate_presigned_post(
+                Bucket=bucket,
+                Key=key,
+                ExpiresIn=expires_in or 3600,
+                Fields={'success_action_status': 200},
+                Conditions=conditions,
+            )
+        elif method == "GET" or method == "PUT":
+            client_method = 'get_object'
+
+            if method == 'PUT':
+                client_method = 'put_object'
+
+            return s3_client.generate_presigned_url(
+                ClientMethod=client_method,
+                ExpiresIn=expires_in,
+                Params={
+                    'Bucket': bucket,
+                    'Key': key
+                },
+            )
+        else:
+            raise ValueError(f"unexpected method: '{method}'")
+
+    @retry_handler
+    def get_multipart_upload_config(self, bucket, key, expires_in=None):
+        sts_client = self._build_client('sts')
+
+        response = sts_client.assume_role(
+            RoleArn=self.aws_role_arn,
+            RoleSessionName='upload_' + str(uuid.uuid4()),
+            DurationSeconds=expires_in,
+            Policy=self._build_upload_polciy(bucket, key)
+        )
+
+        credentials = response["Credentials"]
+        endpoint = self.client._endpoint.host
+        parsed_url = urlparse(endpoint)
+
+        # Multipart upload requires host if format: "Host: Bucket.s3.amazonaws.com"
+        # see https://docs.aws.amazon.com/AmazonS3/latest/API/API_CreateMultipartUpload.html
+        # but Minio requires plain URL, so check netloc and insert bucket only for AWS S3
+        if parsed_url.netloc == AWS_S3_HOST:
+            endpoint = None
+
+        return {
+            "bucket": bucket,
+            "key": key,
+            "config": {
+                "endpoint": endpoint,
+                "port": parsed_url.port or 443,
+                "use_ssl": True,
+                "access_key": credentials["AccessKeyId"],
+                "secret_key": credentials["SecretAccessKey"],
+                "security_token": credentials["SessionToken"],
+            }
+        }
+
+    def _build_upload_polciy(self, bucket, key):
+        return json.dumps(
+            {
+                "Version": '2012-10-17',
+                "Statement": [
+                    {
+                        "Action": [
+                            "s3:*",
+                        ],
+                        "Effect": "Allow",
+                        "Resource": [
+                            f"arn:aws:s3:::{bucket}/{key}"
+                        ],
+                    }
+                ]
             }
         )
 
@@ -134,25 +213,21 @@ class BotoClient:
         return self.client.waiter_names
 
 class S3FSClient:
+    @staticmethod
+    def split_path_to_bucket_and_key(path):
+        uri = urlparse(path)
+
+        rel_path = uri.path[1:]
+        rel_path = rel_path.replace("//", "/")
+        if rel_path.endswith("/"):
+            rel_path = rel_path[:-1]
+
+        return uri.netloc, rel_path
 
     def _get_relative_path(self, path):
-        try:
-            from urllib.parse import urlparse
-        except ImportError:
-            from urlparse import urlparse
-
-        relPath = path
-        uri = urlparse(path)
-        self.s3BucketName = uri.netloc
-
+        self.s3BucketName, rel_path = S3FSClient.split_path_to_bucket_and_key(path)
         self.client = BotoClient()
-        relPath = uri.path[1:]
-
-        relPath = relPath.replace("//", "/")
-        if relPath.endswith("/"):
-            relPath = relPath[:-1]
-
-        return relPath
+        return rel_path
 
     def get_smart_open_transport_params(self):
         transport_params = None
@@ -282,7 +357,7 @@ class S3FSClient:
         path = self._get_relative_path(path)
         path = path + "/" if not path.endswith("/") else path
 
-        self.ensure_bucket_created(self.s3BucketName)
+        #self.ensure_bucket_created(self.s3BucketName)
 
         # If no path we shouldn't create it
         if path != "/":
@@ -308,11 +383,38 @@ class S3FSClient:
             else:
                 raise
 
+        try:
+            config = {
+                'CORSRules': [
+                    {
+                        "AllowedHeaders": ["*"],
+                        "AllowedMethods": ["GET", "PUT", "POST", "DELETE"],
+                        "AllowedOrigins": ["*"],
+                        "ExposeHeaders": ["ETag", "x-amz-request-id", "x-amz-id-2"],
+                        "MaxAgeSeconds": 3600,
+                    }
+                ]
+            }
+
+            self.client.put_bucket_cors(Bucket=Bucket, CORSConfiguration=config)
+        except botocore.exceptions.ClientError as e:
+            if not "NotImplemented" in str(e):
+                raise
+            else:
+                pass
+                # Minio doesn't support CORSes yet https://docs.min.io/docs/minio-server-limits-per-tenant.html
+
     def ensure_bucket_deleted(self, Bucket):
         try:
-            self.client = BotoClient()
-            self.client.head_bucket(Bucket=Bucket)
-            self.client.delete_bucket(Bucket=Bucket)
+            # https://stackoverflow.com/questions/29809105/how-do-i-delete-a-versioned-bucket-in-aws-s3-using-the-cli
+            s3 = boto3.resource(
+                's3',
+                endpoint_url=os.environ.get('S3_ENDPOINT_URL'),
+                config=boto3.session.Config(signature_version='s3v4'),
+            )
+            bucket = s3.Bucket(Bucket)
+            bucket.object_versions.delete()
+            bucket.delete()
         except botocore.client.ClientError as e:
             code = e.response['Error']['Code']
             # 404 - deleted, that how it should work
@@ -494,8 +596,6 @@ class S3FSClient:
             self.download_file(path_src, path_dst)
 
     def _s3_upload_file(self, path_local, path_s3):
-        import boto3
-
         s3_config = boto3.s3.transfer.TransferConfig(use_threads=False)
 
         mimetype, encoding = mimetypes.guess_type(path_local)
@@ -528,7 +628,7 @@ class S3FSClient:
 
     def download_file(self, path, local_path):
         from .local_fsclient import LocalFSClient
-        import boto3
+
         try:
             from urllib.parse import urlparse
         except ImportError:
