@@ -10,39 +10,57 @@ def cmdl(ctx):
     ctx.setup_logger(format='')
 
 @click.command('deploy', short_help='Deploy trained model.')
-@click.argument('model-id', required=True, type=click.STRING)
+@click.argument('model-id', required=False, type=click.STRING)
 @click.option('--locally', is_flag=True, default=False,
     help='Download and deploy trained model locally.')
-@click.option('--no-review', is_flag=True, default=False,
+@click.option('--review', is_flag=True, default=False,
     help='Do not support model review based on actual data.')
-@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
+@click.option('--provider', '-p', type=click.Choice(['auger','azure','external']), required=False,
     help='Cloud AutoML Provider.')
+@click.option('--name', '-n', required=False, type=click.STRING, 
+    help='Model friendly name.Used as name for Review Endpoint')
+@click.option('--algorithm', '-a', required=False, type=click.STRING, 
+    help='Monitored model(external provider) algorithm name.')
+@click.option('--score', '-s', required=False, type=float, 
+    help='Monitored model(external provider) score.')
+@click.option('--data-path', '-d', type=click.STRING, required=False,
+    help='Data path to fit model when deploy. Return new deployed model-id')
 @pass_context
-def deploy(ctx, provider, model_id, locally, no_review):
+def deploy(ctx, provider, model_id, locally, review, name, algorithm, score, data_path):
     """Deploy trained model."""
-    A2MLModel(ctx, provider).deploy(model_id, locally, not no_review)
+    A2MLModel(ctx, provider).deploy(model_id, locally=locally, review=review, 
+        name=name, algorithm=algorithm, score=score, data_path=data_path)
 
 @click.command('predict', short_help='Predict with deployed model.')
+@click.argument('model-id', required=True, type=click.STRING)
 @click.argument('filename', required=True, type=click.STRING)
 @click.option('--threshold', '-t', default=None, type=float,
     help='Threshold.')
-@click.option('--model-id', '-m', type=click.STRING, required=True,
-    help='Deployed model id.')
-@click.option('--locally', is_flag=True, default=False,
+@click.option('--score', '-s', is_flag=True, default=False,
+    help='Calculate scores for predicted results.')
+@click.option('--score_true_path', type=click.STRING, required=False,
+    help='Path to true values to calculate scores. If missed, target from filename used for true values.')
+@click.option('--locally', '-l', is_flag=True, default=False,
+    help='Predict locally using auger.ai.predict package.')
+@click.option('--docker', is_flag=True, default=False,
     help='Predict locally using Docker image to run model.')
 @click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
     help='Cloud AutoML Provider.')
 @click.option('--output', '-o', type=click.STRING, required=False,
     help='Output csv file path.')
 @pass_context
-def predict(ctx, provider, filename, model_id, threshold, locally, output):
+def predict(ctx, provider, filename, model_id, threshold, locally, docker, output):
     """Predict with deployed model."""
-    A2MLModel(ctx, provider).predict(filename=filename, model_id=model_id, threshold=threshold, locally=locally, output=output)
+    if docker:
+        locally = "docker"
+
+    A2MLModel(ctx, provider).predict(model_id, filename=filename, 
+        threshold=threshold, score=score, score_true_data=score_true_path,
+        locally=locally, output=output)
 
 @click.command('actuals', short_help='Send actual values for deployed model. Needed for review and monitoring.')
+@click.argument('model-id', required=True, type=click.STRING)
 @click.argument('filename', required=True, type=click.STRING)
-@click.option('--model-id', '-m', type=click.STRING, required=True,
-    help='Deployed model id.')
 @click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
 help='Cloud AutoML Provider.')
 @click.option('--locally', is_flag=True, default=False,
@@ -56,10 +74,12 @@ def actuals(ctx, provider, filename, model_id, locally):
 @click.argument('model-id', required=True, type=click.STRING)
 @click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
     help='Cloud AutoML Provider.')
+@click.option('--name', '-n', required=False, type=click.STRING, 
+    help='Model friendly name.Used as name for Review Endpoint')
 @pass_context
-def review_alert(ctx, provider, model_id):
+def review_alert(ctx, provider, model_id, name):
     """Predict with deployed model."""
-    A2MLModel(ctx, provider).review_alert(model_id)
+    A2MLModel(ctx, provider).review_alert(model_id, name=name)
 
 @click.command('review', short_help='Review information about deployed model.')
 @click.argument('model-id', required=True, type=click.STRING)
@@ -83,8 +103,7 @@ def undeploy(ctx, provider, model_id, locally):
     A2MLModel(ctx, provider).undeploy(model_id, locally)
 
 @click.command('delete_actuals', short_help='Delete files with actuals and predcitions locally or from specified provider(s).')
-@click.option('--model-id', '-m', type=click.STRING, required=True,
-    help='Deployed model id.')
+@click.argument('model-id', required=True, type=click.STRING)
 @click.option('--with-predictions', is_flag=True, default=False,
     help='Remove predictions.')
 @click.option('--begin-date', '-b', type=click.STRING, required=False,
@@ -97,10 +116,21 @@ help='Cloud AutoML Provider.')
     help='Process actuals locally.')
 @pass_context
 def delete_actuals(ctx, model_id, with_predictions, begin_date, end_date, provider, locally):
-    """Predict with deployed model."""
     A2MLModel(ctx, provider).delete_actuals(model_id, 
         with_predictions=with_predictions, begin_date=begin_date, end_date=end_date, locally=locally)
 
+@click.command('get_info', short_help='Get information about model.')
+@click.argument('model-id', required=True, type=click.STRING)
+@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
+help='Cloud AutoML Provider.')
+@click.option('--locally', is_flag=True, default=False,
+    help='Process actuals locally.')
+@pass_context
+def get_info(ctx, model_id, provider, locally):
+    result = A2MLModel(ctx, provider).get_info(model_id, locally=locally)
+
+    ctx.log('Model information for %s: %s' % (model_id, result))
+
 @pass_context
 def add_commands(ctx):
     cmdl.add_command(deploy)
@@ -110,5 +140,6 @@ def add_commands(ctx):
     cmdl.add_command(review)
     cmdl.add_command(undeploy)
     cmdl.add_command(delete_actuals)
+    cmdl.add_command(get_info)
 
 add_commands()
