@@ -20,7 +20,7 @@ from a2ml.api.stats.feature_divergence import FeatureDivergence
 from a2ml.api.utils import dict_dig, merge_dicts
 from a2ml.api.utils.json_utils import json_dumps_np
 from a2ml.api.utils.context import Context
-from a2ml.api.utils.s3_fsclient import S3FSClient
+from a2ml.api.utils.s3_fsclient import S3FSClient, BotoClient
 from a2ml.tasks_queue.config import Config
 
 from .celery_app import celeryApp
@@ -79,7 +79,7 @@ def build_and_send_task_result_to_hub(args, status, result, traceback=None):
         if params and params.get('hub_info', {}).get('cluster_task_id'):
             response = {
                 'type': 'TaskResult',
-                'provider': params['provider'],
+                'provider': params.get('provider'), # provider == None -> means external provider
                 'hub_info': params['hub_info'],
                 'status': status,
                 'runtime': time.time() - current_task.start_time,
@@ -99,6 +99,7 @@ def process_task_result(task_func):
         try:
             result = task_func(*args, **kwargs)
             build_and_send_task_result_to_hub(args, 'success', result)
+            return result
         except Exception as e:
             build_and_send_task_result_to_hub(
                 args,
@@ -187,8 +188,7 @@ def _read_hub_experiment_session(ctx, params):
         evaluation_options.get('max_n_trials', 100))
     ctx.config.set('experiment/use_ensemble',
         evaluation_options.get('use_ensemble', True))
-    ctx.config.set('experiment/metric',
-        evaluation_options.get('scoring'), provider)
+    ctx.config.set('experiment/metric', evaluation_options.get('scoring'))
 
     if evaluation_options.get('algorithms_to_exlude'):
         ctx.config.set('experiment/blocked_models', evaluation_options.get('algorithms_to_exlude'))
@@ -196,6 +196,12 @@ def _read_hub_experiment_session(ctx, params):
         ctx.config.set('experiment/allowed_models', evaluation_options.get('allowed_algorithms'))
     if evaluation_options.get('exit_score'):
         ctx.config.set('experiment/exit_score', evaluation_options.get('exit_score'))
+    if evaluation_options.get('score_top_count'):
+        ctx.config.set('experiment/score_top_count', evaluation_options.get('score_top_count'))
+    if evaluation_options.get('retrain_policy_type'):
+        ctx.config.set('experiment/retrain_policy_type', evaluation_options.get('retrain_policy_type'))
+    if evaluation_options.get('retrain_policy_value'):
+        ctx.config.set('experiment/retrain_policy_value', evaluation_options.get('retrain_policy_value'))
 
     return ctx
 
@@ -477,12 +483,21 @@ def deploy_model_task(params):
 
     return res
 
+@celeryApp.task(ignore_result=True)
+@process_task_result
+def add_external_model_task(params):
+    return ModelReview(params).add_external_model(
+        target_column=params.get('target_column'),
+        scoring=params.get('scoring'),
+        task_type=params.get('task_type'),
+        binary_classification=params.get('binary_classification', False),
+    )
+
 @celeryApp.task(ignore_result=True)
 @process_task_result
 def undeploy_model_task(params):
     ctx = _create_provider_context(params)
     provider = params.get('provider', 'auger')
-    ctx = _read_hub_experiment_session(ctx, params)
 
     model_id = params.get('hub_info', {}).get('pipeline_id')
     if not model_id:
@@ -492,9 +507,7 @@ def undeploy_model_task(params):
         raise Exception("undeploy_model_task: hub_info/pipeline_id should be provided.")
 
     ctx.config.set('undeploy/service_only', params.get('service_only', False), provider)
-    ctx.config.clean_changes()
     res = A2MLModel(ctx).undeploy(model_id = model_id, locally=params.get('locally', False))
-    _update_hub_objects(ctx, params.get('provider'), params)
 
     return res
 
@@ -518,7 +531,8 @@ def predict_by_model_task(params):
         count_in_result=params.get('count_in_result'),
         predicted_at=params.get('prediction_date'),
         prediction_id = params.get('prediction_id'),
-        locally = params.get('locally', False)
+        locally = params.get('locally', False),
+        no_features_in_result=params.get('no_features_in_result', False)
     )
     _update_hub_objects(ctx, params.get('provider'), params)
 
@@ -527,9 +541,13 @@ def predict_by_model_task(params):
 @celeryApp.task(ignore_result=True)
 @process_task_result
 def score_actuals_by_model_task(params):
-    ctx = _create_provider_context(params)
-    ctx = _read_hub_experiment_session(ctx, params)
-    ctx.config.clean_changes()
+    ctx = None
+    external_model = params.get("external_model", False)
+
+    if not external_model:
+        ctx = _create_provider_context(params)
+        ctx = _read_hub_experiment_session(ctx, params)
+        ctx.config.clean_changes()
 
     return ModelReview(params).add_actuals(
         ctx,
@@ -540,7 +558,10 @@ def score_actuals_by_model_task(params):
         actual_date_column=params.get('actual_date_column'),
         actuals_id=params.get('actuals_id'),
         return_count=params.get('return_count', False),
-        provider=params.get('provider')
+        provider=params.get('provider'),
+        external_model=external_model,
+        do_predict=params.get('do_predict', False),
+        experiment_params=params.get('experiment_params'),
     )
 
 @celeryApp.task(ignore_result=True)
@@ -555,9 +576,21 @@ def delete_actuals_task(params):
 @celeryApp.task(ignore_result=True)
 @process_task_result
 def score_model_performance_daily_task(params):
+    ctx = None
+    external_model = params.get("external_model", False)
+
+    if not external_model:
+        ctx = _create_provider_context(params)
+        ctx = _read_hub_experiment_session(ctx, params)
+        ctx.config.clean_changes()
+
     return ModelReview(params).score_model_performance_daily(
         date_from=params.get('date_from'),
-        date_to=params.get('date_to')
+        date_to=params.get('date_to'),
+        extra_features=params.get("features", []),
+        provider=params.get('provider'),
+        do_predict=params.get('do_predict'),
+        ctx=ctx
     )
 
 @celeryApp.task(ignore_result=True)
@@ -572,7 +605,7 @@ def set_support_review_model_flag_task(params):
 def distribution_chart_stats_task(params):
     return ModelReview(params).distribution_chart_stats(
         date_from=params.get('date_from'),
-        date_to=params.get('date_to')
+        date_to=params.get('date_to'),
     )
 
 @celeryApp.task(ignore_result=True)
@@ -595,9 +628,15 @@ def build_review_data_task(params):
     if ctx.config.get_list('experiment/date_time'):
         date_col = ctx.config.get_list('experiment/date_time')[0]
 
+    retrain_policy = None
+    if ctx.config.get('experiment/retrain_policy_type'):
+        retrain_policy = {'type': ctx.config.get('experiment/retrain_policy_type'), 'value':ctx.config.get('experiment/retrain_policy_value')}
+
     return ModelReview(params).build_review_data(
         data_path=params.get('data_path'),
-        date_col = date_col
+        date_col = date_col,
+        retrain_policy = retrain_policy,
+        date_to=params.get('date_to')
     )
 
 @celeryApp.task(ignore_result=True)
@@ -640,3 +679,35 @@ def delete_org_bucket(params):
     client.ensure_bucket_deleted(Bucket=params['bucket_name'])
 
     return True
+
+@celeryApp.task(ignore_result=True)
+@process_task_result
+def presign_s3_url_task(params):
+    client = BotoClient(region=params.get('region'))
+
+    path = params.get('path')
+    if path:
+        params['bucket'], params['key'] = S3FSClient.split_path_to_bucket_and_key(path)
+
+    if params.get('multipart', False):
+        return client.get_multipart_upload_config(
+            bucket=params['bucket'],
+            key=params['key'],
+            expires_in=params.get('expires_in'),
+        )
+    else:
+        return client.generate_presigned_url_ex(
+            bucket=params['bucket'],
+            key=params['key'],
+            method=params['method'],
+            expires_in=params.get('expires_in'),
+            max_content_length=params.get('max_content_length'),
+        )
+
+@celeryApp.task(ignore_result=True)
+@process_task_result
+def validate_roi_syntax_task(params):
+    return ModelReview(params).validate_roi_syntax(
+        expressions=params["expressions"],
+        features=params.get("features", []),
+    )
